
# THIS FILE IS GENERATED FROM syned SETUP.PY
short_version = '1.0.17'
version = '1.0.17'
full_version = '1.0.17.dev0+ac41b0e'
git_revision = 'ac41b0eaeb5546558a764f6d9c4b3ea95b52f3d7'
release = False

if not release:
    version = full_version
    short_version += ".dev"
